from .args import subparsers, load_arguments
from .arguments.dns import parser_dns
from .arguments.letsencrypt import parser_letsencrypt
from .arguments.server import parser_server
from .arguments.ip import parser_ip

loaded = True


import pydantic
import json
import logging
import urllib.error

from ..output import log
from ..helpers import post_request, get_request


class IP(pydantic.BaseModel):
	base_endpoint = "/ip"

	def listfree(self):
		from ..session import configuration
		from ..cli.args import args
		
		log(f"Listing all IP's accessible by user {configuration.credentials.user}", fg="gray", level=logging.INFO)
		try:
			data = post_request(f'{self.base_endpoint}/listfree', {
				"ipversion": args.ipversion,
				"datacenter": args.datacenter,
				"platform": args.platform
			})
		except urllib.error.HTTPError as error:
			if error.status == 400:
				data = json.loads(error.read().decode())
			else:
				raise error

		if (status := data.get('response', {}).get('status', {}).get('code')) == 200:
			return data.get('response', {}).get('iplist', [])
		elif status == 400:
			reason = data.get('response', {}).get('status', {}).get('text')
			log(f"Could not get free IP's: {reason.strip()}", fg="orange", level=logging.ERROR)
			return {}


from ..args import subparsers
from ..ip import ip_entrypoint

parser_ip = subparsers.add_parser("ip", help="Perform IP actions on Glesys API")
parser_ip.add_argument(
	"--list-free",
	required=False,
	action="store_true",
	default=False,
	help="Lists all available IP's to the user",
)
parser_ip.add_argument(
	"--ipversion",
	required=True,
	type=str,
	help="During --list-free which IP version are we looking for? 4 or 6?",
)
parser_ip.add_argument(
	"--datacenter",
	required=True,
	type=str,
	help="During --list-free which Datacenter are we looking for free IP's in?",
)
parser_ip.add_argument(
	"--platform",
	required=True,
	type=str,
	help="During --list-free which platform are we querying (KVM, OpenVZ, VMware)",
)

parser_ip.set_defaults(func=ip_entrypoint)


import pydantic

from .dns import DNS
from .server import Server
from .ip import IP


class API(pydantic.BaseModel):
	"""
	This class serves as a router/gateway to the
	different endpoints in the API.

	/domain/list for instance lives under api.dns
	"""
	user :str
	key :str
	host :str

	dns :DNS = DNS()
	server :Server = Server()
	ip :IP = IP()


import json

from ..output import log
from ..session import api


def server_entrypoint(args, **kwargs):
	if args.all_servers:
		print(json.dumps(api.server.list(), indent=4))

	elif args.find:
		servers = api.server.find(
			hostname=args.name,
			description=args.description,
			datacenter=args.datacenter,
			platform=args.platform,
			iplist=args.ip,
			state=args.state,
			lazy=args.lazy
		)

		for server in servers:
			print(json.dumps(server, indent=4))

	elif args.details:
		print(json.dumps(api.server.details(args.details), indent=4))
	elif args.adapters:
		print(json.dumps(api.server.networkadapters(args.adapters), indent=4))
	elif args.status:
		print(json.dumps(api.server.status(args.status), indent=4))
	elif args.limits:
		print(json.dumps(api.server.limits(args.limits), indent=4))
	elif args.costs:
		print(json.dumps(api.server.costs(args.costs), indent=4))
	elif args.est_costs:
		print(json.dumps(api.server.estimatedcost(args.est_costs), indent=4))
	


import urllib.request
import json
import base64

def get_request(endpoint):
	from .session import configuration

	credentials = base64.b64encode(f"{configuration.credentials.user}:{configuration.credentials.key}".encode())

	req = urllib.request.Request(f"{configuration.network.host}{endpoint}")
	req.add_header('Accept', 'application/json; charset=utf-8')
	req.add_header('Authorization', f'Basic {credentials.decode()}')

	response = urllib.request.urlopen(req)
	response_data = json.loads(response.read().decode())

	return response_data


def post_request(endpoint, payload):
	from .session import configuration

	credentials = base64.b64encode(f"{configuration.credentials.user}:{configuration.credentials.key}".encode())

	req = urllib.request.Request(f"{configuration.network.host}{endpoint}")
	req.add_header('Content-Type', 'application/json; charset=utf-8')
	req.add_header('Authorization', f'Basic {credentials.decode()}')

	json_payload = json.dumps(payload).encode('utf-8')
	
	req.add_header('Content-Length', len(json_payload))
	response = urllib.request.urlopen(req, json_payload)

	response_data = json.loads(response.read().decode())

	return response_data


import pydantic
import json
import logging
import urllib.error

from ..output import log
from ..helpers import post_request, get_request


class Server(pydantic.BaseModel):
	base_endpoint = "/server"

	def _fuzzy_find(self, server):
		if not (server_info := list(self.find(hostname=server, server_id=server))):
			raise ValueError(f"Could not find server with name={server} or server_id={name}")

		if len(server_info) > 1:
			raise ValueError(f"Multiple servers found with the name/server_id of {server}")
		else:
			server_info = server_info[0]

		return server_info

	def list(self):
		from ..session import configuration
		
		log(f"Listing all servers accessible by user {configuration.credentials.user}", fg="gray", level=logging.INFO)
		data = get_request(f'{self.base_endpoint}/list')

		if data.get('response', {}).get('status', {}).get('code') == 200:
			return data.get('response', {}).get('servers', [])

	def find(self, lazy=False, **kwargs):
		"""
		Due to the amount of properties on a server, this function
		can take any ambigious top-level property of the server detail listing
		and match on anonymous parameters via **kwargs.

		For instance:
		api.server.find(description="some description") will look in the
		/server/list result for any server containing info.get('description') == "some description"

		Adding --lazy to the search will perform:
		"some description" in info.get('description') instead
		"""
		from ..cli.args import args

		log(f"Trying to locate server based on search parameters lazy={lazy} and {kwargs}", fg="gray", level=logging.INFO)

		for server in self.list():
			for key in kwargs:
				if kwargs[key] is None:
					continue

				if lazy:
					if kwargs[key].lower() in str(server.get(key)):
						yield server
						break
				if str(server.get(key)).lower() == kwargs[key].lower():
					yield server
					break

	def details(self, server):
		server_info = self._fuzzy_find(server)

		data = get_request(f"{self.base_endpoint}/details?serverid={server_info['serverid']}")

		if data.get('response', {}).get('status', {}).get('code') == 200:
			return data.get('response', {}).get('server', {})

	def networkadapters(self, server):
		server_info = self._fuzzy_find(server)

		data = get_request(f"{self.base_endpoint}/networkadapters?serverid={server_info['serverid']}")

		if data.get('response', {}).get('status', {}).get('code') == 200:
			return data.get('response', {}).get('networkadapters', {})

	def status(self, server):
		server_info = self._fuzzy_find(server)

		data = get_request(f"{self.base_endpoint}/status?serverid={server_info['serverid']}")

		if data.get('response', {}).get('status', {}).get('code') == 200:
			return data.get('response', {}).get('server', {})

	def limits(self, server):
		server_info = self._fuzzy_find(server)

		try:
			data = get_request(f"{self.base_endpoint}/limits?serverid={server_info['serverid']}")
		except urllib.error.HTTPError as error:
			if error.status == 400:
				data = json.loads(error.read().decode())
			else:
				raise error

		if (status := data.get('response', {}).get('status', {}).get('code')) == 200:
			return data.get('response', {}).get('server', {})
		elif status == 400:
			reason = data.get('response', {}).get('status', {}).get('text')
			log(f"Could not get limits of this server: {reason}", fg="gray", level=logging.INFO)
			return {}

	def costs(self, server):
		server_info = self._fuzzy_find(server)

		data = get_request(f"{self.base_endpoint}/costs?serverid={server_info['serverid']}")

		if data.get('response', {}).get('status', {}).get('code') == 200:
			return data.get('response', {}).get('costs', {})

	def estimatedcost(self, server):
		"""
		This API endpoint takes an existing serverid or
		a server spec and convert that to an estimated cost.
		"""

		return {} # Returns 400 Bad Request currently

		server_info = self._fuzzy_find(server)

		data = post_request(f"{self.base_endpoint}/estimatedcost", {'serverid' : server_info['serverid']})

		if data.get('response', {}).get('status', {}).get('code') == 200:
			return data.get('response', {}).get('estimatedcost', {})

	


from ..args import subparsers
from ..server import server_entrypoint

parser_server = subparsers.add_parser("server", help="Perform Server actions on Glesys API")
parser_server.add_argument(
	"--all-servers",
	required=False,
	action="store_true",
	default=False,
	help="Lists all available servers to the user",
)
parser_server.add_argument(
	"--find",
	required=False,
	action="store_true",
	default=False,
	help="Attempts to find a server given --name, --description, --datacenter, --platform, --ip or --state",
)
parser_server.add_argument(
	"--lazy",
	required=False,
	action="store_true",
	default=False,
	help="Attempts a more relaced .find() by matching more loosely (mandatory for searching --ip for instance)",
)
parser_server.add_argument(
	"--details",
	required=False,
	default=None,
	type=str,
	help="Prints the details of a server (takes server_id or hostname/name)",
)
parser_server.add_argument(
	"--adapters",
	required=False,
	default=None,
	type=str,
	help="Prints the network adapters of a given server (takes server_id or hostname/name)",
)
parser_server.add_argument(
	"--status",
	required=False,
	default=None,
	type=str,
	help="Prints the status of a given server (takes server_id or hostname/name)",
)
parser_server.add_argument(
	"--limits",
	required=False,
	default=None,
	type=str,
	help="Prints the OpenVZ limits of a given server (takes server_id or hostname/name)",
)
parser_server.add_argument(
	"--costs",
	required=False,
	default=None,
	type=str,
	help="Prints the costs of a given server (takes server_id or hostname/name)",
)
parser_server.add_argument(
	"--est-costs",
	required=False,
	default=None,
	type=str,
	help="Prints the estimated costs of a given server (takes server_id or hostname/name)",
)


parser_server.add_argument(
	"--name",
	required=False,
	default=None,
	type=str,
	help="This is a search parameter for --find",
)
parser_server.add_argument(
	"--description",
	required=False,
	default=None,
	type=str,
	help="This is a search parameter for --find",
)
parser_server.add_argument(
	"--datacenter",
	required=False,
	default=None,
	type=str,
	help="This is a search parameter for --find",
)
parser_server.add_argument(
	"--platform",
	required=False,
	default=None,
	type=str,
	help="This is a search parameter for --find",
)
parser_server.add_argument(
	"--ip",
	required=False,
	default=None,
	type=str,
	help="This is a search parameter for --find",
)
parser_server.add_argument(
	"--state",
	required=False,
	default=None,
	type=str,
	help="This is a search parameter for --find",
)

parser_server.set_defaults(func=server_entrypoint)